@@ -2,7 +2,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2001, 2002, 2003,
-              2004, 2005, 2006, 2007 John W. Eaton
+              2004, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -228,24 +228,7 @@ public:
 
   virtual ~Array (void);
 
-  Array<T>& operator = (const Array<T>& a)
-    {
-      if (this != &a)
-	{
-	  if (--rep->count <= 0)
-	    delete rep;
-
-	  rep = a.rep;
-	  rep->count++;
-
-	  dimensions = a.dimensions;
-
-	  idx_count = 0;
-	  idx = 0;
-	}
-
-      return *this;
-    }
+  Array<T>& operator = (const Array<T>& a);
 
   void fill (const T& val) { make_unique (val); }
 
@@ -516,6 +499,8 @@ public:
 
   void maybe_delete_elements (Array<idx_vector>& ra_idx, const T& rfv);
 
+  void maybe_delete_elements (octave_idx_type dim, idx_vector& i);
+
   Array<T> value (void) const;
 
   Array<T> index (idx_vector& i, int resize_ok = 0,
