@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+              2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -63,8 +63,7 @@ char file_ops::dir_sep_char = '/';
 std::string file_ops::dir_sep_str ("/");
 #endif
 
-#if (defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM) \
-     && defined (OCTAVE_HAVE_POSIX_FILESYSTEM))
+#if defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM)
 std::string file_ops::dir_sep_chars ("/\\");
 #else
 std::string file_ops::dir_sep_chars (file_ops::dir_sep_str);
@@ -644,7 +643,7 @@ tilde_find_suffix (const std::string& s)
 
   for ( ; i < s_len; i++)
     {
-      if (s[i] == file_ops::dir_sep_char)
+      if (file_ops::is_dir_sep (s[i]))
 	break;
 
       if (! suffixes.empty ())
@@ -671,7 +670,7 @@ isolate_tilde_prefix (const std::string& fname)
 
   size_t len = 1;
 
-  while (len < f_len && fname[len] != file_ops::dir_sep_char)
+  while (len < f_len && ! file_ops::is_dir_sep (fname[len]))
     len++;
 
   return fname.substr (1, len);
@@ -692,7 +691,7 @@ tilde_expand_word (const std::string& filename)
   // of $HOME or the home directory of the current user, regardless of
   // any preexpansion hook.
 
-  if (f_len == 1 || filename[1] == file_ops::dir_sep_char)
+  if (f_len == 1 || file_ops::is_dir_sep (filename[1]))
     return octave_env::get_home_directory () + filename.substr (1);
 
   std::string username = isolate_tilde_prefix (filename);
@@ -858,6 +857,16 @@ file_ops::is_dir_sep (char c)
   return dir_sep_chars.find (c) != NPOS;
 }
 
+std::string
+file_ops::concat (const std::string& dir, const std::string& file)
+{
+  return dir.empty ()
+    ? file
+    : (is_dir_sep (dir[dir.length()-1])
+       ? dir + file
+       : dir + file_ops::dir_sep_char + file);
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
