@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 2004 John W. Eaton
+Copyright (C) 2004, 2005, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -30,17 +29,19 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 typedef intNDArray<octave_int32> int32NDArray;
 
-NDS_CMP_OP_DECLS (int32NDArray, octave_int32)
-NDS_BOOL_OP_DECLS (int32NDArray, octave_int32)
+NDS_CMP_OP_DECLS (int32NDArray, octave_int32, OCTAVE_API)
+NDS_BOOL_OP_DECLS (int32NDArray, octave_int32, OCTAVE_API)
 
-SND_CMP_OP_DECLS (octave_int32, int32NDArray)
-SND_BOOL_OP_DECLS (octave_int32, int32NDArray)
+SND_CMP_OP_DECLS (octave_int32, int32NDArray, OCTAVE_API)
+SND_BOOL_OP_DECLS (octave_int32, int32NDArray, OCTAVE_API)
 
-NDND_CMP_OP_DECLS (int32NDArray, int32NDArray)
-NDND_BOOL_OP_DECLS (int32NDArray, int32NDArray)
+NDND_CMP_OP_DECLS (int32NDArray, int32NDArray, OCTAVE_API)
+NDND_BOOL_OP_DECLS (int32NDArray, int32NDArray, OCTAVE_API)
 
 MARRAY_FORWARD_DEFS (MArrayN, int32NDArray, octave_int32)
 
+MINMAX_DECLS (int32)
+
 #endif
 
 /*
