@@ -1,4 +1,4 @@
-## Copyright (C) 2006, 2007 John W. Eaton
+## Copyright (C) 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -28,8 +28,17 @@
 ## the common size of @var{alpha} and @var{sigma}.
 ## @end deftypefn
 
+## Deprecated in version 3.0
+
 function rnd = weibrnd (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "weibrnd is obsolete and will be removed from a future version of Octave; please use wblrnd instead");
+  endif
+
   rnd = wblrnd (varargin{:});
 
 endfunction
