@@ -20,9 +20,9 @@
 
 ## Author: jwe
 
-function __go_draw_figure__ (f, plot_stream)
+function __go_draw_figure__ (f, plot_stream, enhanced, mono)
 
-  if (nargin == 2)
+  if (nargin == 4)
     if (strcmp (f.type, "figure"))
 
       ## Set figure properties here?
@@ -37,10 +37,15 @@ function __go_draw_figure__ (f, plot_stream)
 	  switch (obj.type)
 	    case "axes"
 	      axes_count++;
+	      ## Force multiplot with a colorbar to ensure colorbar on the page
+	      if (!strcmp (obj.__colorbar__, "none"))
+		axes_count++;
+	      endif
 	  endswitch
 	endfor
 
 	fputs (plot_stream, "\nreset;\n");
+	fputs (plot_stream, "set autoscale fix;\n");
 
 	multiplot_mode = axes_count > 1;
 
@@ -52,8 +57,7 @@ function __go_draw_figure__ (f, plot_stream)
 	  obj = get (kids(i));
 	  switch (obj.type)
 	    case "axes"
-	      __go_draw_axes__ (kids(i), plot_stream);
-
+	      __go_draw_axes__ (kids (i), plot_stream, enhanced, mono);
 	    otherwise
 	      error ("__go_draw_figure__: unknown object class, %s",
 		     obj.type);
