@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 2005, 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -43,6 +43,10 @@ function drawnow (term, file, mono, debug_file)
     endif
 
     if (nargin >= 2 && nargin <= 4)
+      [dnm, fnm, ext] = fileparts (file);
+      if (! (isempty (dnm) || isdir (dnm)))
+	error ("drawnow: nonexistent directory `%s'", dnm);
+      endif
       h = get (0, "currentfigure");
       if (h)
 	f = get (h);
@@ -208,7 +212,7 @@ function have_enhanced = enhanced_term (term)
     ## should not be interpreted in that case.
     if (compare_versions (__gnuplot_version__ (), "4.0", ">"))
       enhanced_terminals = {"aqua", "dumb", "png", "jpeg", "gif", "pm", ...
-	                    "windows", "wxt", "svg", "postscript", "x11"};
+	                    "windows", "wxt", "svg", "postscript", "x11", "pdf"};
     else 
       enhanced_terminals = {"x11", "postscript"};
     endif
