@@ -23,13 +23,20 @@
 ## @end deftypefn
 
 function s = spones (s)
+
+  if (nargin != 1)
+    print_usage ();
+  endif
+
   if (issparse (s))
     [i, j, v, m, n] = spfind (s);
   else
     [i, j, v] = find (s);
     [m, n] = size (s);
-  end
+  endif
+
   s = sparse (i, j, 1, m, n);
+
 endfunction
 
 %!assert(issparse(spones([1,2;3,0])))
