@@ -48,16 +48,17 @@ function [count, mean, var] = spstats (S, j)
   count = spsum (sparse (i, j, 1, n, m));
   if (nargout > 1) 
     mean = spsum (S) ./ count; 
-  end
+  endif
   if (nargout > 2) 
     ## FIXME Variance with count = 0 or 1?
     diff = S - sparse (i, j, mean(j), n, m); 
     var = spsum (diff .* diff) ./ (count - 1);
-  end
+  endif
+
 endfunction
 
 %!test
 %! [n,m,v] = spstats([1 2 1 2 3 4],[2 2 1 1 1 1]);
-%! assert(n,[4,2]);
-%! assert(m,[10/4,3/2],10*eps);
-%! assert(v,[5/3,1/2],10*eps);
+%! assert(n,sparse([4,2]));
+%! assert(m,sparse([10/4,3/2]),10*eps);
+%! assert(v,sparse([5/3,1/2]),10*eps);
