@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 2005 Nicolo' Giorgetti
+Copyright (C) 2005, 2006, 2007 Nicolo' Giorgetti
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -40,7 +39,11 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 extern "C"
 {
+#if defined (HAVE_GLPK_GLPK_H)
+#include <glpk/glpk.h>
+#else
 #include <glpk.h>
+#endif
 
 #ifdef GLPK_PRE_4_14
 
@@ -192,7 +195,12 @@ glpk (int sense, int n, int m, double *c, int nz, int *rn, int *cn,
     {
       //-- Define type of the structural variables
       if (! freeLB[i] && ! freeUB[i])
-	lpx_set_col_bnds (lp, i+1, LPX_DB, lb[i], ub[i]);
+	{
+	  if (lb[i] != ub[i])
+	    lpx_set_col_bnds (lp, i+1, LPX_DB, lb[i], ub[i]);
+	  else
+	    lpx_set_col_bnds (lp, i+1, LPX_FX, lb[i], ub[i]);
+	}
       else
 	{
 	  if (! freeLB[i] && freeUB[i])
@@ -451,8 +459,7 @@ glpk (int sense, int n, int m, double *c, int nz, int *rn, int *cn,
 DEFUN_DLD (__glpk__, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {[@var{values}] =} __glpk__ (@var{args})\n\
-Internal interface for the GNU GLPK library.\n\
-You should be using using the @code{glpk} function instead.\n\
+Undocumented internal function.\n\
 @end deftypefn")
 {
   // The list of values to return.  See the declaration in oct-obj.h
@@ -568,7 +575,7 @@ You should be using using the @code{glpk} function instead.\n\
   //--            bound on each of the variables.
   Matrix LB (args(3).matrix_value ());
 
-  if (error_state)
+  if (error_state || LB.length () < mrowsc)
     {
       error ("__glpk__: invalid value of lb");
       return retval;
@@ -593,7 +600,7 @@ You should be using using the @code{glpk} function instead.\n\
   //--            bound on each of the variables.
   Matrix UB (args(4).matrix_value ());
 
-  if (error_state)
+  if (error_state || UB.length () < mrowsc)
     {
       error ("__glpk__: invalid value of ub");
       return retval;
@@ -794,11 +801,11 @@ You should be using using the @code{glpk} function instead.\n\
   OCTAVE_GLPK_GET_REAL_PARAM ("tolobj", 9);
 
   //-- Assign pointers to the output parameters
-  ColumnVector xmin (mrowsc);
-  ColumnVector fmin (1);
+  ColumnVector xmin (mrowsc, octave_NA);
+  ColumnVector fmin (1, octave_NA);
   ColumnVector status (1);
-  ColumnVector lambda (mrowsA);
-  ColumnVector redcosts (mrowsc);
+  ColumnVector lambda (mrowsA, octave_NA);
+  ColumnVector redcosts (mrowsc, octave_NA);
   ColumnVector time (1);
   ColumnVector mem (1);
 
