@@ -1,14 +1,14 @@
 /*
 
-Copyright (C) 2004 David Bateman
+Copyright (C) 1997, 1999, 2002, 2004, 2005, 2006, 2007 David Bateman
 Copyright (C) 1996, 1997 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -49,7 +48,7 @@ do_fft (const octave_value_list &args, const char *fcn, int type)
 
   if (nargin < 1 || nargin > 3)
     {
-      print_usage (fcn);
+      print_usage ();
       return retval;
     }
 
@@ -99,7 +98,7 @@ do_fft (const octave_value_list &args, const char *fcn, int type)
   if (dim < 0)
     {
       for (octave_idx_type i = 0; i < dims.length (); i++)
-	if ( dims(i) > 1)
+	if (dims(i) > 1)
 	  {
 	    dim = i;
 	    break;
@@ -115,8 +114,8 @@ do_fft (const octave_value_list &args, const char *fcn, int type)
   else
     dims (dim) = n_points;
 
-  if (dims.all_zero () || n_points == 0)
-    return octave_value (Matrix ());
+  if (dims.any_zero () || n_points == 0)
+    return octave_value (NDArray (dims));
 
   if (arg.is_real_type ())
     {
@@ -146,6 +145,20 @@ do_fft (const octave_value_list &args, const char *fcn, int type)
   return retval;
 }
 
+/*
+
+%!error(fft())
+%!assert(fft([]), [])
+%!assert(fft(zeros(10,0)), zeros(10,0))
+%!assert(fft(zeros(0,10)), zeros(0,10))
+%!assert(fft(0), 0)
+%!assert(fft(1), 1)
+%!assert(fft(1), 1)
+%!assert(fft(ones(2,2)), [2,2; 0,0])
+%!assert(fft(eye(2,2)), [1,1; 1,-1])
+
+*/
+
 
 DEFUN_DLD (fft, args, ,
   "-*- texinfo -*-\n\
@@ -160,14 +173,14 @@ If called with two arguments, @var{n} is expected to be an integer\n\
 specifying the number of elements of @var{a} to use, or an empty\n\
 matrix to specify that its value should be ignored. If @var{n} is\n\
 larger than the dimension along which the FFT is calculated, then\n\
-@var{a} is resized and padded with zeros. Otherwise, if@var{n} is\n\
+@var{a} is resized and padded with zeros. Otherwise, if @var{n} is\n\
 smaller than the dimension along which the FFT is calculated, then\n\
 @var{a} is truncated.\n\
 \n\
-If called with three agruments, @var{dim} is an integer specifying the\n\
+If called with three arguments, @var{dim} is an integer specifying the\n\
 dimension of the matrix along which the FFT is performed\n\
-@end deftypefn\n\
-@seealso {ifft, fft2, fftn, fftw_wisdom}")
+@seealso{ifft, fft2, fftn, fftw}\n\
+@end deftypefn")
 {
   return do_fft (args, "fft", 0);
 }
@@ -190,10 +203,10 @@ larger than the dimension along which the inverse FFT is calculated, then\n\
 smaller than the dimension along which the inverse FFT is calculated,\n\
 then @var{a} is truncated.\n\
 \n\
-If called with three agruments, @var{dim} is an integer specifying the\n\
+If called with three arguments, @var{dim} is an integer specifying the\n\
 dimension of the matrix along which the inverse FFT is performed\n\
-@end deftypefn\n\
-@seealso {fft, ifft2, ifftn, fftw_wisdom}")
+@seealso{fft, ifft2, ifftn, fftw}\n\
+@end deftypefn")
 {
   return do_fft (args, "ifft", 1);
 }
