@@ -471,8 +471,9 @@ DEFUN_DLD (randn, args, ,
 @deftypefnx {Loadable Function} {} randn (@var{n}, @var{m})\n\
 @deftypefnx {Loadable Function} {} randn (\"state\", @var{x})\n\
 @deftypefnx {Loadable Function} {} randn (\"seed\", @var{x})\n\
-Return a matrix with normally distributed random elements. The\n\
-arguments are handled the same as the arguments for @code{rand}.\n\
+Return a matrix with normally distributed pseudo-random\n\
+elements having zero mean and variance one. The arguments are\n\
+handled the same as the arguments for @code{rand}.\n\
 \n\
 By default, @code{randn} uses a Marsaglia and Tsang Ziggurat technique to\n\
 transform from a uniform to a normal distribution. (G. Marsaglia and\n\
@@ -950,7 +951,7 @@ D 50 p1284, 1994\n\
 %!test
 %! % Test fixed state
 %! randp("seed",1);
-%! assert(randp(1e9,1,6),[1000006208 1000012224 999981120 999963520 999963072 999981440])
+%! assert(randp(1e9,1,6),[1000006208 1000012224 999981120 999963520 999963072 999981440],1e-6)
 %!test
 %! if (__random_statistical_tests__)
 %!   % statistical tests may fail occasionally.
