@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007 John W. Eaton
+              2002, 2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -53,11 +53,11 @@ static bool Vbeep_on_error = false;
 // TRUE means that Octave will try to enter the debugger when an error
 // is encountered.  This will also inhibit printing of the normal
 // traceback message (you will only see the top-level error message).
-static bool Vdebug_on_error = false;
+bool Vdebug_on_error = false;
 
 // TRUE means that Octave will try to enter the debugger when a warning
 // is encountered.
-static bool Vdebug_on_warning = false;
+bool Vdebug_on_warning = false;
 
 // TRUE means that Octave will try to display a stack trace when a
 // warning is encountered.
@@ -620,7 +620,12 @@ warning_enabled (const std::string& id)
 	retval = all_state;
     }
   else if (all_state == 2)
-    retval = 2;
+    {
+      if (id_state == 0)
+	retval= id_state;
+      else
+	retval = all_state;
+    }
 
   return retval;
 }
@@ -958,6 +963,7 @@ location of the error. Typically @var{err} is returned from\n\
 DEFUN (error, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} error (@var{template}, @dots{})\n\
+@deftypefnx {Built-in Function} {} error (@var{id}, @var{template}, @dots{})\n\
 Format the optional arguments under the control of the template string\n\
 @var{template} using the same rules as the @code{printf} family of\n\
 functions (@pxref{Formatted Output}) and print the resulting message\n\
@@ -1011,18 +1017,43 @@ error: nargin != 1\n\
 @end example\n\
 @end deftypefn")
 {
-  // FIXME -- need to extract and pass message id to
-  // handle_message.
+  octave_value retval;
+
+  int nargin = args.length ();
+
+  octave_value_list nargs = args;
+
+  std::string id;
+
+  if (nargin > 1)
+    {
+      std::string arg1 = args(0).string_value ();
+
+      if (! error_state)
+	{
+	  if (arg1.find ('%') == NPOS)
+	    {
+	      id = arg1;
+
+	      nargs.resize (nargin-1);
+
+	      for (int i = 1; i < nargin; i++)
+		nargs(i-1) = args(i);
+	    }
+	}
+      else
+	return retval;
+    }
+
+  handle_message (error_with_id, id.c_str (), "unspecified error", nargs);
 
-  octave_value_list retval;
-  handle_message (error_with_id, "", "unspecified error", args);
   return retval;
 }
 
 DEFCMD (warning, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} warning (@var{template}, @dots{})\n\
-@deftypefnx {Built-in Function} {} warning (@var{id}, @var{template})\n\
+@deftypefnx {Built-in Function} {} warning (@var{id}, @var{template}, @dots{})\n\
 Format the optional arguments under the control of the template string\n\
 @var{template} using the same rules as the @code{printf} family of\n\
 functions (@pxref{Formatted Output}) and print the resulting message\n\
@@ -1074,8 +1105,50 @@ warning named by @var{id} is handled as if it were an error instead.\n\
 		{
 		  Octave_map tmp;
 
-		  tmp.assign ("identifier", arg2);
-		  tmp.assign ("state", arg1);
+		  Cell id (1, 1);
+		  Cell st (1, 1);
+
+		  id(0) = arg2;
+		  st(0) = arg1;
+
+		  // Since internal Octave functions are not
+		  // compatible, turning all warnings into errors
+		  // should leave the state of
+		  // Octave:matlab-incompatible alone.
+
+		  if (arg1 == "error"
+		      && warning_options.contains ("identifier"))
+		    {
+		      octave_idx_type n = 1;
+
+		      Cell tid = warning_options.contents ("identifier");
+		      Cell tst = warning_options.contents ("state");
+
+		      for (octave_idx_type i = 0; i < tid.numel (); i++)
+			{
+			  octave_value vid = tid(i);
+
+			  if (vid.is_string ())
+			    {
+			      std::string key = vid.string_value ();
+
+			      if (key == "Octave:matlab-incompatible"
+				  || key == "Octave:single-quote-string")
+				{
+				  id.resize (dim_vector (1, n+1));
+				  st.resize (dim_vector (1, n+1));
+
+				  id(n) = tid(i);
+				  st(n) = tst(i);
+
+				  n++;
+				}
+			    }
+			}
+		    }
+
+		  tmp.assign ("identifier", id);
+		  tmp.assign ("state", st);
 
 		  warning_options = tmp;
 
@@ -1296,8 +1369,6 @@ warning named by @var{id} is handled as if it were an error instead.\n\
 	    return retval;
 	}
 
-      // handle_message.
-
       std::string prev_msg = Vlast_warning_message;
 
       std::string curr_msg = handle_message (warning_with_id, id.c_str (),
@@ -1387,6 +1458,11 @@ their default values.\n\
   octave_value retval;
   int nargin = args.length();
 
+  unwind_protect::begin_frame ("Flasterror");
+
+  unwind_protect_int (error_state);
+  error_state = 0;
+
   if (nargin < 2)
     {
       Octave_map err;
@@ -1511,6 +1587,8 @@ their default values.\n\
   else
     print_usage ();
 
+  unwind_protect::run_frame ("Flasterror");
+
   return retval;  
 }
 
@@ -1524,6 +1602,11 @@ also set the last message identifier.\n\
 {
   octave_value_list retval;
 
+  unwind_protect::begin_frame ("Flasterr");
+
+  unwind_protect_int (error_state);
+  error_state = 0;
+
   int argc = args.length () + 1;
 
   if (argc < 4)
@@ -1553,6 +1636,8 @@ also set the last message identifier.\n\
   else
     print_usage ();
 
+  unwind_protect::run_frame ("Flasterr");
+
   return retval;  
 }
 
