@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-              2005, 2006, 2007 John W. Eaton
+              2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,6 +26,7 @@ along with Octave; see the file COPYING.  If not, see
 #endif
 
 #include <cassert>
+#include <cctype>
 #include <cstring>
 
 #include <iomanip>
@@ -1039,12 +1040,6 @@ octave_base_stream::gets (octave_idx_type max_len, bool& err, const std::string&
   return do_gets (max_len, err, false, who);
 }
 
-#if defined (__GNUG__) && ! defined (CXX_ISO_COMPLIANT_LIBRARY)
-
-#define OCTAVE_SCAN(is, fmt, arg) is.scan ((fmt).text, arg)
-
-#else
-
 #define OCTAVE_SCAN(is, fmt, arg) octave_scan (is, fmt, arg)
 
 template <class T>
@@ -1071,14 +1066,18 @@ octave_scan_1 (std::istream& is, const scanf_format_elt& fmt, T* valptr)
 	  {
 	    if (c1 == '0')
 	      {
-		int c2 = is.get ();
+		int c2 = is.peek ();
 
 		if (c2 == 'x' || c2 == 'X')
-		  is >> std::hex >> ref >> std::dec;
+                  {
+                    is.ignore ();
+                    if (std::isxdigit (is.peek ()))
+                      is >> std::hex >> ref >> std::dec;
+                    else
+                      ref = 0;
+                  }
 		else
 		  {
-		    is.putback (c2);
-
 		    if (c2 == '0' || c2 == '1' || c2 == '2'
 			|| c2 == '3' || c2 == '4' || c2 == '5'
 			|| c2 == '6' || c2 == '7')
@@ -1339,8 +1338,6 @@ octave_scan<> (std::istream& is, const scanf_format_elt& fmt, double* valptr)
   return is;
 }
 
-#endif
-
 template <class T>
 void
 do_scanf_conv (std::istream& is, const scanf_format_elt& fmt,
@@ -1547,45 +1544,37 @@ do_scanf_conv (std::istream&, const scanf_format_elt&, double*,
  \
   do \
     { \
-      if (width) \
-	{ \
-	  char *tbuf = new char[width+1]; \
+      if (! width) \
+	width = INT_MAX; \
  \
-	  OCTAVE_SCAN (is, *elt, tbuf); \
+      std::ostringstream buf; \
  \
-	  tbuf[width] = '\0'; \
-          tmp = tbuf; \
-          delete [] tbuf; \
+      std::string char_class = elt->char_class; \
+ \
+      int c = EOF; \
+ \
+      if (elt->type == '[') \
+        { \
+	  int chars_read = 0; \
+	  while (is && chars_read++ < width && (c = is.get ()) != EOF \
+	         && char_class.find (c) != NPOS) \
+	    buf << static_cast<char> (c); \
 	} \
       else \
 	{ \
-	  std::ostringstream buf; \
- \
-	  std::string char_class = elt->char_class; \
- \
-	  int c = EOF; \
- \
-	  if (elt->type == '[') \
-	    { \
-	      while (is && (c = is.get ()) != EOF \
-		     && char_class.find (c) != NPOS) \
-		buf << static_cast<char> (c);	     \
-	    } \
-	  else \
-	    { \
-	      while (is && (c = is.get ()) != EOF \
-		     && char_class.find (c) == NPOS) \
-		buf << static_cast<char> (c);		     \
-	    } \
- \
-	  if (c != EOF) \
-	    is.putback (c); \
- \
-	  tmp = buf.str (); \
- \
-	  if (tmp.empty ()) \
-	    is.setstate (std::ios::failbit); \
+	  int chars_read = 0; \
+	  while (is && chars_read++ < width && (c = is.get ()) != EOF \
+	         && char_class.find (c) == NPOS) \
+	    buf << static_cast<char> (c); \
 	} \
+ \
+      if (width == INT_MAX && c != EOF) \
+	is.putback (c); \
+ \
+      tmp = buf.str (); \
+ \
+      if (tmp.empty ()) \
+        is.setstate (std::ios::failbit); \
     } \
   while (0)
 
@@ -3138,13 +3127,15 @@ do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx
 		  elts_read++;
 		}
 
+	      int seek_status = 0;
+
 	      if (skip != 0 && elts_read == block_size)
 		{
-		  strm.seek (skip, SEEK_CUR);
+		  seek_status = strm.seek (skip, SEEK_CUR);
 		  elts_read = 0;
 		}
 
-	      if (is.eof ())
+	      if (is.eof () || seek_status < 0)
 		{
 		  if (nr > 0)
 		    {
@@ -3261,6 +3252,10 @@ octave_stream::read (const Array<double>& size, octave_idx_type block_size,
       FILL_TABLE_ROW (oct_data_conv::dt_uint32, uint32NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_int64, int64NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_uint64, uint64NDArray);
+      // FIXME -- the following line allows things like int8=>single
+      // to work, but they will actually return a double value.  We
+      // need a floatNDArray for this to work properly.
+      FILL_TABLE_ROW (oct_data_conv::dt_single, NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_double, NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_char, charNDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_schar, charNDArray);
