@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1998 John W. Eaton
+Copyright (C) 1998, 2000, 2002, 2003, 2004, 2005, 2006, 2007
+              John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -31,6 +31,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include "mx-base.h"
 #include "str-vec.h"
+#include "MatrixType.h"
 
 #include "error.h"
 #include "oct-obj.h"
@@ -50,22 +51,22 @@ octave_base_matrix : public octave_base_value
 public:
 
   octave_base_matrix (void)
-    : octave_base_value () { }
+    : octave_base_value (), typ (MatrixType ()) { }
 
-  octave_base_matrix (const MT& m)
-    : octave_base_value (), matrix (m)
+  octave_base_matrix (const MT& m, const MatrixType& t = MatrixType ())
+    : octave_base_value (), matrix (m), typ (t)
   {
     if (matrix.ndims () == 0)
       matrix.resize (dim_vector (0, 0));
   }
 
   octave_base_matrix (const octave_base_matrix& m)
-    : octave_base_value (), matrix (m.matrix) { }
+    : octave_base_value (), matrix (m.matrix), typ (m.typ) { }
 
   ~octave_base_matrix (void) { }
 
-  octave_value *clone (void) const { return new octave_base_matrix (*this); }
-  octave_value *empty_clone (void) const { return new octave_base_matrix (); }
+  octave_base_value *clone (void) const { return new octave_base_matrix (*this); }
+  octave_base_value *empty_clone (void) const { return new octave_base_matrix (); }
 
   size_t byte_size (void) const { return matrix.byte_size (); }
 
@@ -85,27 +86,30 @@ public:
 			 const std::list<octave_value_list>& idx,
 			 const octave_value& rhs);
 
-  octave_value do_index_op (const octave_value_list& idx, int resize_ok);
-
-  octave_value do_index_op (const octave_value_list& idx)
-    { return do_index_op (idx, 0); }
+  octave_value do_index_op (const octave_value_list& idx,
+			    bool resize_ok = false);
 
   void assign (const octave_value_list& idx, const MT& rhs);
 
   dim_vector dims (void) const { return matrix.dims (); }
 
+  octave_idx_type nnz (void) const { return matrix.nnz (); }
+
   octave_value reshape (const dim_vector& new_dims) const
     { return MT (matrix.reshape (new_dims)); }
 
   octave_value permute (const Array<int>& vec, bool inv = false) const
     { return MT (matrix.permute (vec, inv)); }
 
-  octave_value resize (const dim_vector& dv) const
-    { MT retval (matrix); retval.resize (dv); return retval; }
+  octave_value resize (const dim_vector& dv, bool fill = false) const;
 
   octave_value all (int dim = 0) const { return matrix.all (dim); }
   octave_value any (int dim = 0) const { return matrix.any (dim); }
 
+  MatrixType matrix_type (void) const { return typ; }
+  MatrixType matrix_type (const MatrixType& _typ) const
+    { MatrixType ret = typ; typ = _typ; return ret; }
+
   bool is_matrix_type (void) const { return true; }
 
   bool is_numeric_type (void) const { return true; }
@@ -122,9 +126,15 @@ public:
 
   void print_info (std::ostream& os, const std::string& prefix) const;
 
+  // Unsafe.  This function exists to support the MEX interface.
+  // You should not use it anywhere else.
+  void *mex_get_data (void) const { return matrix.mex_get_data (); }
+
 protected:
 
   MT matrix;
+
+  mutable MatrixType typ;
 };
 
 #endif
