@@ -1,21 +1,22 @@
 /*
 
-Copyright (C) 2004 David Bateman
+Copyright (C) 2004, 2005, 2006, 2007 David Bateman
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This file is part of Octave.
 
-This program is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
+Octave is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
+
+Octave is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 In addition to the terms of the GPL, you are permitted to link
 this program with any Open Source program, as defined by the
@@ -29,6 +30,7 @@ Open Source Initiative (www.opensource.org)
 
 #include <istream>
 #include <iostream>
+#include <sstream>
 #include <vector>
 
 #include "defun.h"
@@ -50,7 +52,7 @@ DEFINE_OCTAVE_ALLOCATOR (octave_fcn_inline);
 
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_fcn_inline,
 				     "inline function",
-				     "inline function");
+				     "function_handle");
 
 octave_fcn_inline::octave_fcn_inline (const std::string& f,
 				      const string_vector& a,
@@ -59,7 +61,7 @@ octave_fcn_inline::octave_fcn_inline (const std::string& f,
 {
   // Form a string representing the function.
 
-  OSSTREAM buf;
+  std::ostringstream buf;
 
   buf << "@(";
 
@@ -71,13 +73,10 @@ octave_fcn_inline::octave_fcn_inline (const std::string& f,
       buf << ifargs(i);
     }
 
-  buf << ") " << iftext << OSSTREAM_ENDS;
+  buf << ") " << iftext;
 
   int parse_status;
-  octave_value anon_fcn_handle = eval_string (OSSTREAM_STR (buf), true,
-					      parse_status);
-
-  OSSTREAM_FREEZE (buf);
+  octave_value anon_fcn_handle = eval_string (buf.str (), true, parse_status);
 
   if (parse_status == 0)
     {
@@ -91,8 +90,28 @@ octave_fcn_inline::octave_fcn_inline (const std::string& f,
     error ("inline: unable to define function");
 }
 
+// This function is supplied to allow a Matlab style class structure
+// to be returned..
+Octave_map
+octave_fcn_inline::map_value (void) const
+{
+  Octave_map m;
+  string_vector args = fcn_arg_names ();
+  m.assign ("version", octave_value (1.0));
+  m.assign ("isEmpty", octave_value (0.0));
+  m.assign ("expr", octave_value (fcn_text ()));
+  m.assign ("numArgs", octave_value (args.length ()));
+  m.assign ("args", octave_value (args));
+  std::ostringstream buf;
+  for (int i = 0; i < args.length (); i++)
+    buf << args(i) << " = INLINE_INPUTS_{" << i + 1 << "}; ";
+  m.assign ("inputExpr", octave_value (buf.str ()));
+
+  return m;
+}
+
 bool
-octave_fcn_inline::save_ascii (std::ostream& os, bool&, bool)
+octave_fcn_inline::save_ascii (std::ostream& os)
 {
   os << "# nargs: " <<  ifargs.length () << "\n";
   for (int i = 0; i < ifargs.length (); i++)
@@ -120,7 +139,7 @@ octave_fcn_inline::load_ascii (std::istream& is)
 	nm = "";
 
       char c;
-      OSSTREAM buf;
+      std::ostringstream buf;
 
       // Skip preceeding newline(s)
       while (is.get (c) && c == '\n');
@@ -140,9 +159,7 @@ octave_fcn_inline::load_ascii (std::istream& is)
 	    }
 	}
 
-      buf << OSSTREAM_ENDS;
-      iftext = OSSTREAM_STR (buf);
-      OSSTREAM_FREEZE (buf);
+      iftext = buf.str ();
 
       octave_fcn_inline tmp (iftext, ifargs, nm);
       fcn = tmp.fcn;
@@ -156,19 +173,19 @@ octave_fcn_inline::load_ascii (std::istream& is)
 bool
 octave_fcn_inline::save_binary (std::ostream& os, bool&)
 {
-  FOUR_BYTE_INT tmp = ifargs.length ();
-  os.write (X_CAST (char *, &tmp), 4);
+  int32_t tmp = ifargs.length ();
+  os.write (reinterpret_cast<char *> (&tmp), 4);
   for (int i = 0; i < ifargs.length (); i++)
     {
       tmp = ifargs(i).length ();
-      os.write (X_CAST (char *, &tmp), 4);
+      os.write (reinterpret_cast<char *> (&tmp), 4);
       os.write (ifargs(i).c_str (), ifargs(i).length ());
     }
   tmp = nm.length ();
-  os.write (X_CAST (char *, &tmp), 4);
+  os.write (reinterpret_cast<char *> (&tmp), 4);
   os.write (nm.c_str (), nm.length ());
   tmp = iftext.length ();
-  os.write (X_CAST (char *, &tmp), 4);
+  os.write (reinterpret_cast<char *> (&tmp), 4);
   os.write (iftext.c_str (), iftext.length ());
   return true;
 }
@@ -177,8 +194,8 @@ bool
 octave_fcn_inline::load_binary (std::istream& is, bool swap,
 				oct_mach_info::float_format)
 {
-  FOUR_BYTE_INT nargs;
-  if (! is.read (X_CAST (char *, &nargs), 4))
+  int32_t nargs;
+  if (! is.read (reinterpret_cast<char *> (&nargs), 4))
     return false;
   if (swap)
     swap_bytes<4> (&nargs);
@@ -187,11 +204,11 @@ octave_fcn_inline::load_binary (std::istream& is, bool swap,
     return false;
   else
     {
-      FOUR_BYTE_INT tmp;
+      int32_t tmp;
       ifargs.resize (nargs);
       for (int i = 0; i < nargs; i++)
 	{
-	  if (! is.read (X_CAST (char *, &tmp), 4))
+	  if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	    return false;
 	  if (swap)
 	    swap_bytes<4> (&tmp);
@@ -204,7 +221,7 @@ octave_fcn_inline::load_binary (std::istream& is, bool swap,
 	    return false;
 	}
 
-      if (! is.read (X_CAST (char *, &tmp), 4))
+      if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&tmp);
@@ -216,7 +233,7 @@ octave_fcn_inline::load_binary (std::istream& is, bool swap,
       if (! is)
 	return false;
 
-      if (! is.read (X_CAST (char *, &tmp), 4))
+      if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&tmp);
@@ -251,7 +268,7 @@ octave_fcn_inline::save_hdf5 (hid_t loc_id, const char *name,
   hid_t space_hid = -1, data_hid = -1, type_hid = -1;;
   bool retval = true;
 
-  // XXX FIXME XXX Is there a better way of saving string vectors, than a
+  // FIXME Is there a better way of saving string vectors, than a
   // null padded matrix?
 
   OCTAVE_LOCAL_BUFFER (hsize_t, hdims, 2);
@@ -309,7 +326,7 @@ octave_fcn_inline::save_hdf5 (hid_t loc_id, const char *name,
     }
 
   hdims[0] = 0;
-  space_hid = H5Screate_simple (0 , hdims, (hsize_t*) 0);
+  space_hid = H5Screate_simple (0 , hdims, 0);
   if (space_hid < 0)
     {
       H5Tclose (type_hid);
@@ -319,7 +336,7 @@ octave_fcn_inline::save_hdf5 (hid_t loc_id, const char *name,
 
   data_hid = H5Dcreate (group_hid, "nm",  type_hid, space_hid, H5P_DEFAULT);
   if (data_hid < 0 || H5Dwrite (data_hid, type_hid, H5S_ALL, H5S_ALL,
-				H5P_DEFAULT, (void*) nm.c_str ()) < 0)
+				H5P_DEFAULT, nm.c_str ()) < 0)
     {
       H5Sclose (space_hid);
       H5Tclose (type_hid);
@@ -339,7 +356,7 @@ octave_fcn_inline::save_hdf5 (hid_t loc_id, const char *name,
   data_hid = H5Dcreate (group_hid, "iftext",  type_hid, space_hid,
 			H5P_DEFAULT);
   if (data_hid < 0 || H5Dwrite (data_hid, type_hid, H5S_ALL, H5S_ALL,
-				H5P_DEFAULT, (void*) iftext.c_str ()) < 0)
+				H5P_DEFAULT, iftext.c_str ()) < 0)
     {
       H5Sclose (space_hid);
       H5Tclose (type_hid);
@@ -449,8 +466,7 @@ octave_fcn_inline::load_hdf5 (hid_t loc_id, const char *name,
   st_id = H5Tcopy (H5T_C_S1);
   H5Tset_size (st_id, slen);
 
-  if (H5Dread (data_hid, st_id, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-	       X_CAST (void *, nm_tmp)) < 0)
+  if (H5Dread (data_hid, st_id, H5S_ALL, H5S_ALL, H5P_DEFAULT, nm_tmp) < 0)
     {
       H5Sclose (space_hid);
       H5Tclose (type_hid);
@@ -508,8 +524,7 @@ octave_fcn_inline::load_hdf5 (hid_t loc_id, const char *name,
   st_id = H5Tcopy (H5T_C_S1);
   H5Tset_size (st_id, slen);
 
-  if (H5Dread (data_hid, st_id, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-	       X_CAST (void *, iftext_tmp)) < 0)
+  if (H5Dread (data_hid, st_id, H5S_ALL, H5S_ALL, H5P_DEFAULT, iftext_tmp) < 0)
     {
       H5Sclose (space_hid);
       H5Tclose (type_hid);
@@ -537,7 +552,7 @@ octave_fcn_inline::print (std::ostream& os, bool pr_as_read_syntax) const
 void
 octave_fcn_inline::print_raw (std::ostream& os, bool pr_as_read_syntax) const
 {
-  OSSTREAM buf;
+  std::ostringstream buf;
 
   if (nm.empty ())
     buf << "f(";
@@ -552,11 +567,10 @@ octave_fcn_inline::print_raw (std::ostream& os, bool pr_as_read_syntax) const
       buf << ifargs(i);
     }
 
-  buf << ") = " << iftext << OSSTREAM_ENDS;
+  buf << ") = " << iftext;
 
-  octave_print_internal (os, OSSTREAM_STR (buf), pr_as_read_syntax,
+  octave_print_internal (os, buf.str (), pr_as_read_syntax,
 			 current_print_indent_level ());
-  OSSTREAM_FREEZE (buf);
 }
 
 octave_value
@@ -565,10 +579,10 @@ octave_fcn_inline::convert_to_str_internal (bool, bool, char type) const
   return octave_value (fcn_text (), type);
 }
 
-DEFUN (inline, args, ,
+DEFUNX ("inline", Finline, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} inline (@var{str})\n\
-@deftypefnx {Built-in Function} {} inline (@var{str}, @var{arg1}, ...)\n\
+@deftypefnx {Built-in Function} {} inline (@var{str}, @var{arg1}, @dots{})\n\
 @deftypefnx {Built-in Function} {} inline (@var{str}, @var{n})\n\
 Create an inline function from the character string @var{str}.\n\
 If called with a single argument, the arguments of the generated\n\
@@ -576,7 +590,7 @@ function are extracted from the function itself. The generated\n\
 function arguments will then be in alphabetical order. It should\n\
 be noted that i, and j are ignored as arguments due to the\n\
 ambiguity between their use as a variable or their use as an inbuilt\n\
-constant. All arguments followed by a parentheses are considered\n\
+constant. All arguments followed by a parenthesis are considered\n\
 to be functions.\n\
 \n\
 If the second and subsequent arguments are character strings,\n\
@@ -584,8 +598,8 @@ they are the names of the arguments of the function.\n\
 \n\
 If the second argument is an integer @var{n}, the arguments are\n\
 @code{\"x\"}, @code{\"P1\"}, @dots{}, @code{\"P@var{N}\"}.\n\
-@end deftypefn\n\
-@seealso{argnames, formula, vectorize}")
+@seealso{argnames, formula, vectorize}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -659,7 +673,11 @@ If the second argument is an integer @var{n}, the arguments are\n\
 			    break;
 			  }
 			  
-		      if (! have_arg && tmp_arg != "i" && tmp_arg != "j")
+		      if (! have_arg && tmp_arg != "i" && tmp_arg != "j" &&
+			  tmp_arg != "NaN" && tmp_arg != "nan" && 
+			  tmp_arg != "Inf" && tmp_arg != "inf" && 
+			  tmp_arg != "NA" && tmp_arg != "pi" &&
+			  tmp_arg != "eps")
 			fargs.append (tmp_arg);
 
 		      tmp_arg = std::string ();
@@ -684,10 +702,9 @@ If the second argument is an integer @var{n}, the arguments are\n\
 
 		      for (int i = 1; i < n+1; i++)
 			{
-			  OSSTREAM buf;
-			  buf << "P" << i << OSSTREAM_ENDS;
-			  fargs(i) = OSSTREAM_STR (buf);
-			  OSSTREAM_FREEZE (buf);
+			  std::ostringstream buf;
+			  buf << "P" << i;
+			  fargs(i) = buf.str ();
 			}
 		    }
 		  else
@@ -726,7 +743,7 @@ If the second argument is an integer @var{n}, the arguments are\n\
 	error ("inline: first argument must be a string");
     }
   else
-    print_usage ("inline");
+    print_usage ();
 
   return retval;
 }
@@ -737,8 +754,8 @@ DEFUN (formula, args, ,
 Return a character string representing the inline function @var{fun}.\n\
 Note that @code{char (@var{fun})} is equivalent to\n\
 @code{formula (@var{fun})}.\n\
-@end deftypefn\n\
-@seealso{argnames, inline, vectorize}")
+@seealso{argnames, inline, vectorize}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -754,7 +771,7 @@ Note that @code{char (@var{fun})} is equivalent to\n\
 	error ("formula: must be an inline function");
     }
   else
-    print_usage ("formula");
+    print_usage ();
 
   return retval;
 }
@@ -764,8 +781,8 @@ DEFUN (argnames, args, ,
 @deftypefn {Built-in Function} {} argnames (@var{fun})\n\
 Return a cell array of character strings containing the names of\n\
 the arguments of the inline function @var{fun}.\n\
-@end deftypefn\n\
-@seealso{argnames, inline, formula, vectorize}")
+@seealso{inline, formula, vectorize}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -790,19 +807,18 @@ the arguments of the inline function @var{fun}.\n\
 	error ("argnames: argument must be an inline function");
     }
   else
-    print_usage ("argnames");
+    print_usage ();
 
   return retval;
 }
 
 DEFUN (vectorize, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} argnames (@var{fun})\n\
+@deftypefn {Built-in Function} {} vectorize (@var{fun})\n\
 Create a vectorized version of the inline function @var{fun}\n\
 by replacing all occurrences of @code{*}, @code{/}, etc., with\n\
 @code{.*}, @code{./}, etc.\n\
-@end deftypefn\n\
-@seealso{argnames, inline, formula, vectorize}")
+@end deftypefn")
 {
   octave_value retval;
 
@@ -810,13 +826,26 @@ by replacing all occurrences of @code{*}, @code{/}, etc., with\n\
 
   if (nargin == 1)
     {
-      octave_fcn_inline* old = args(0).fcn_inline_value (true);
+      std::string old_func;
+      octave_fcn_inline* old = 0;
+      bool func_is_string = true;
 
-      if (old)
+      if (args(0).is_string ())
+	old_func = args(0).string_value ();
+      else
 	{
-	  std::string old_func = old->fcn_text ();
-	  std::string new_func;
+	  old = args(0).fcn_inline_value (true);
+	  func_is_string = false;
 
+	  if (old)
+	    old_func = old->fcn_text ();
+	  else
+	    error ("vectorize: must be a string or inline function");
+	}
+
+      if (! error_state)
+	{
+	  std::string new_func;
 	  size_t i = 0;
 
 	  while (i < old_func.length ())
@@ -840,13 +869,15 @@ by replacing all occurrences of @code{*}, @code{/}, etc., with\n\
 	      i++;
 	    }
 
-	  retval = octave_value (new octave_fcn_inline (new_func, old->fcn_arg_names ()));
+	  if (func_is_string)
+	    retval = octave_value (new_func);
+	  else
+	    retval = octave_value (new octave_fcn_inline 
+				   (new_func, old->fcn_arg_names ()));
 	}
-      else
-	error ("vectorize: must be an inline function");
     }
   else
-    print_usage ("vectorize");
+    print_usage ();
 
   return retval;
 }
