@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+              2002, 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -40,18 +40,12 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <unistd.h>
 #endif
 
-// Include setjmp.h, not csetjmp since the latter might only define
-// the ANSI standard C interface.
-
-#include <setjmp.h>
-
 #include "quit.h"
 
 #include "dir-ops.h"
 #include "file-ops.h"
 #include "file-stat.h"
 #include "lo-mappers.h"
-#include "lo-sstream.h"
 #include "oct-cmplx.h"
 #include "oct-env.h"
 #include "pathsearch.h"
@@ -64,6 +58,8 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "error.h"
 #include "gripes.h"
 #include "input.h"
+#include "load-path.h"
+#include "oct-errno.h"
 #include "oct-hist.h"
 #include "oct-obj.h"
 #include "pager.h"
@@ -73,12 +69,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "utils.h"
 #include "variables.h"
 
-// If TRUE, print a warning for expressions like
-//
-//   ones (-1, 5)
-//
-static int Vwarn_neg_dim_as_zero;
-
 // Return TRUE if S is a valid identifier.
 
 bool
@@ -118,11 +108,19 @@ Return true if @var{name} is a valid variable name\n\
   if (argc == 2)
     retval = valid_identifier (argv[1]);
   else
-    print_usage ("isvarname");
+    print_usage ();
 
   return retval;
 }
 
+// Return TRUE if F and G are both names for the same file.
+
+bool
+same_file (const std::string& f, const std::string& g)
+{
+  return same_file_internal (f, g);
+}
+
 int
 almost_match (const std::string& std, const std::string& s, int min_match_len,
 	      int case_sens)
@@ -255,9 +253,9 @@ search_path_for_all_files (const std::string& path, const string_vector& names)
 
   string_vector sv = p.find_all_first_of (names);
 
-  int len = sv.length ();
+  octave_idx_type len = sv.length ();
 
-  for (int i = 0; i < len; i++)
+  for (octave_idx_type i = 0; i < len; i++)
     sv[i] = octave_env::make_absolute (sv[i], octave_env::getcwd ());
 
   return sv;
@@ -266,12 +264,14 @@ search_path_for_all_files (const std::string& path, const string_vector& names)
 static string_vector
 make_absolute (const string_vector& sv)
 {
-  int len = sv.length ();
+  octave_idx_type len = sv.length ();
 
-  for (int i = 0; i < len; i++)
-    sv[i] = octave_env::make_absolute (sv[i], octave_env::getcwd ());
+  string_vector retval (len);
 
-  return sv;
+  for (octave_idx_type i = 0; i < len; i++)
+    retval[i] = octave_env::make_absolute (sv[i], octave_env::getcwd ());
+ 
+  return retval;
 }
 
 DEFUN (file_in_loadpath, args, ,
@@ -279,19 +279,19 @@ DEFUN (file_in_loadpath, args, ,
 @deftypefn {Built-in Function} {} file_in_loadpath (@var{file})\n\
 @deftypefnx {Built-in Function} {} file_in_loadpath (@var{file}, \"all\")\n\
 \n\
-Return the absolute name name of @var{file} if it can be found in\n\
-the list of directories specified by @code{LOADPATH}.\n\
+Return the absolute name of @var{file} if it can be found in\n\
+the list of directories specified by @code{path}.\n\
 If no file is found, return an empty matrix.\n\
 \n\
-If the first argument is a cell array of of strings, search each\n\
+If the first argument is a cell array of strings, search each\n\
 directory of the loadpath for element of the cell array and return\n\
 the first that matches.\n\
 \n\
 If the second optional argument @code{\"all\"} is supplied, return\n\
 a cell array containing the list of all files that have the same\n\
 name in the path.  If no files are found, return an empty cell array.\n\
-@end deftypefn\n\
-@seealso{file_in_path}")
+@seealso{file_in_path, path}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -306,8 +306,7 @@ name in the path.  If no files are found, return an empty cell array.\n\
 	  if (nargin == 1)
 	    {
 	      std::string fname = octave_env::make_absolute
-		(Vload_path_dir_path.find_first_of (names),
-		 octave_env::getcwd ());
+		(load_path::find_first_of (names), octave_env::getcwd ());
 
 	      if (fname.empty ())
 		retval = Matrix ();
@@ -319,7 +318,7 @@ name in the path.  If no files are found, return an empty cell array.\n\
 	      std::string opt = args(1).string_value ();
 
 	      if (! error_state && opt == "all")
-		retval = Cell (make_absolute (Vload_path_dir_path.find_all_first_of (names)));
+		retval = Cell (make_absolute (load_path::find_all_first_of (names)));
 	      else
 		error ("file_in_loadpath: invalid option");
 	    }
@@ -328,7 +327,7 @@ name in the path.  If no files are found, return an empty cell array.\n\
 	error ("file_in_loadpath: expecting string as first argument");
     }
   else
-    print_usage ("file_in_loadpath");
+    print_usage ();
 
   return retval;
 }
@@ -337,18 +336,17 @@ DEFUN (file_in_path, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} file_in_path (@var{path}, @var{file})\n\
 @deftypefnx {Built-in Function} {} file_in_path (@var{path}, @var{file}, \"all\")\n\
-Return the absolute name name of @var{file} if it can be found in\n\
+Return the absolute name of @var{file} if it can be found in\n\
 @var{path}.  The value of @var{path} should be a colon-separated list of\n\
-directories in the format described for the built-in variable\n\
-@code{LOADPATH}.  If no file is found, return an empty matrix.\n\
-For example,\n\
+directories in the format described for @code{path}.  If no file\n\
+is found, return an empty matrix.  For example,\n\
 \n\
 @example\n\
-file_in_path (LOADPATH, \"nargchk.m\")\n\
-     @result{} \"@value{OCTAVEHOME}/share/octave/2.0/m/general/nargchk.m\"\n\
+file_in_path (EXEC_PATH, \"sh\")\n\
+     @result{} \"/bin/sh\"\n\
 @end example\n\
 \n\
-If the second argument is a cell array of of strings, search each\n\
+If the second argument is a cell array of strings, search each\n\
 directory of the path for element of the cell array and return\n\
 the first that matches.\n\
 \n\
@@ -398,7 +396,7 @@ name in the path.  If no files are found, return an empty cell array.\n\
 	error ("file_in_path: expecting string as first argument");
     }
   else
-    print_usage ("file_in_path");
+    print_usage ();
 
   return retval;
 }
@@ -411,8 +409,8 @@ file_in_path (const std::string& name, const std::string& suffix)
   if (! suffix.empty ())
     nm.append (suffix);
 
-  return octave_env::make_absolute (Vload_path_dir_path.find (nm),
-				    octave_env::getcwd ());
+  return octave_env::make_absolute
+    (load_path::find_file (nm), octave_env::getcwd ());
 }
 
 // See if there is an function file in the path.  If so, return the
@@ -427,16 +425,23 @@ fcn_file_in_path (const std::string& name)
   
   if (len > 0)
     {
-      if (len > 2 && name [len - 2] == '.' && name [len - 1] == 'm')
-	retval = file_in_path (name, "");
+      if (octave_env::absolute_pathname (name))
+	{
+	  file_stat fs (name);
+
+	  if (fs.exists ())
+	    retval = name;
+	}
+      else if (len > 2 && name [len - 2] == '.' && name [len - 1] == 'm')
+	retval = load_path::find_fcn_file (name.substr (0, len-2));
       else
-	retval = file_in_path (name, ".m");
+	retval = load_path::find_fcn_file (name);
     }
 
   return retval;
 }
 
-// See if there is an octave file in the path.  If so, return the
+// See if there is a .oct file in the path.  If so, return the
 // full path to the file.
 
 std::string
@@ -448,11 +453,47 @@ oct_file_in_path (const std::string& name)
   
   if (len > 0)
     {
-      if (len > 2 && name [len - 4] == '.' && name [len - 3] == 'o'
-	  && name [len - 2] == 'c' && name [len - 1] == 't')
-	retval = file_in_path (name, "");
+      if (octave_env::absolute_pathname (name))
+	{
+	  file_stat fs (name);
+
+	  if (fs.exists ())
+	    retval = name;
+	}
+      else if (len > 4 && name [len - 4] == '.' && name [len - 3] == 'o'
+	       && name [len - 2] == 'c' && name [len - 1] == 't')
+	retval = load_path::find_oct_file (name.substr (0, len-4));
       else
-	retval = file_in_path (name, ".oct");
+	retval = load_path::find_oct_file (name);
+    }
+
+  return retval;
+}
+
+// See if there is a .mex file in the path.  If so, return the
+// full path to the file.
+
+std::string
+mex_file_in_path (const std::string& name)
+{
+  std::string retval;
+
+  int len = name.length ();
+  
+  if (len > 0)
+    {
+      if (octave_env::absolute_pathname (name))
+	{
+	  file_stat fs (name);
+
+	  if (fs.exists ())
+	    retval = name;
+	}
+      else if (len > 4 && name [len - 4] == '.' && name [len - 3] == 'm'
+	       && name [len - 2] == 'e' && name [len - 1] == 'x')
+	retval = load_path::find_mex_file (name.substr (0, len-4));
+      else
+	retval = load_path::find_mex_file (name);
     }
 
   return retval;
@@ -560,7 +601,7 @@ Convert special characters in @var{string} to their escaped forms.\n\
 	error ("do_string_escapes: argument must be a string");
     }
   else
-    print_usage ("do_string_escapes");
+    print_usage ();
 
   return retval;
 }
@@ -664,104 +705,100 @@ representation.\n\
 	error ("undo_string_escapes: argument must be a string");
     }
   else
-    print_usage ("undo_string_escapes");
+    print_usage ();
 
   return retval;
 }
 
-DEFUN (find_first_of_in_loadpath, args, , "")
+DEFUNX ("errno", Ferrno, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{err} =} errno ()\n\
+@deftypefnx {Built-in Function} {@var{err} =} errno (@var{val})\n\
+@deftypefnx {Built-in Function} {@var{err} =} errno (@var{name})\n\
+Return the current value of the system-dependent variable errno,\n\
+set its value to @var{val} and return the previous value, or return\n\
+the named error code given @var{name} as a character string, or -1\n\
+if @var{name} is not found.\n\
+@end deftypefn")
 {
   octave_value retval;
 
-  if (args.length () == 1)
-    {
-      string_vector names = args(0).all_strings ();
+  int nargin = args.length ();
 
-      if (! error_state)
-	retval = Vload_path_dir_path.find_all_first_of (names);
+  if (nargin == 1)
+    {
+      if (args(0).is_string ())
+	{
+	  std::string nm = args(0).string_value ();
+
+	  if (! error_state)
+	    retval = octave_errno::lookup (nm);
+	  else
+	    error ("errno: expecting character string argument");
+	}
+      else
+	{
+	  int val = args(0).int_value ();
+
+	  if (! error_state)
+	    retval = octave_errno::set (val);
+	  else
+	    error ("errno: expecting integer argument");
+	}
     }
+  else if (nargin == 0)
+    retval = octave_errno::get ();
   else
-    print_usage ("find_first_of_in_loadpath");
+    print_usage ();
 
   return retval;
 }
 
+// See if there is a directory called "name" in the path and if it
+// contains a Contents.m file return the full path to this file.
 
-// #if 0
+std::string
+contents_file_in_path (const std::string& dir)
+{
+  std::string retval;
 
-// Octave could use some way to access the value of ERRNO, but this is
-// probably not the best interface, so don't depend on it...
+  if (dir.length () > 0)
+    {
+      std::string tcontents = file_ops::concat (load_path::find_dir (dir), 
+						std::string ("Contents.m"));
 
-DEFUN (ERRNO, args, ,
+      file_stat fs (tcontents);
+
+      if (fs.exists ())
+	retval = octave_env::make_absolute (tcontents, octave_env::getcwd ());
+    }
+
+  return retval;
+}
+
+DEFUN (errno_list, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {@var{system_error_number}} errno ()\n\
-Return the current value of the system-dependent variable errno.\n\
+@deftypefn {Built-in Function} {} errno_list ()\n\
+Return a structure containing the system-dependent errno values.\n\
 @end deftypefn")
 {
   octave_value retval;
 
   if (args.length () == 0)
-    retval = errno;
+    retval = octave_errno::list ();
   else
-    print_usage ("errno");
+    print_usage ();
 
   return retval;
 }
 
-// #endif
-
-static void
-warn_old_style_preference (bool val, const std::string& sval)
-{
-  warning
-    ("preference of \"%s\" is obsolete -- use numeric value of %d instead",
-     sval.c_str (), (val ? 1 : 0));
-}
-
-// Check the value of a string variable to see if it it's ok to do
-// something.
-//
-//   return of  1 => always ok.
-//   return of  0 => never ok.
-//   return of -1 => ok, but give me warning (default).
-
-int
-check_preference (const std::string& var)
-{
-  int pref = -1;
-
-  std::string val = builtin_string_variable (var);
-
-  if (val.empty ())
-    {
-      double dval = 0;
-      if (builtin_real_scalar_variable (var, dval))
-	pref = NINT (dval);
-    }
-  else
-    {
-      if (val == "yes" || val == "true")
-	{
-	  warn_old_style_preference (true, val);
-	  pref = 1;
-	}
-      else if (val == "never" || val == "no" || val == "false")
-	{
-	  warn_old_style_preference (false, val);
-	  pref = 0;
-	}
-    }
-
-  return pref;
-}
-
 static void
 check_dimensions (octave_idx_type& nr, octave_idx_type& nc, const char *warnfor)
 {
   if (nr < 0 || nc < 0)
     {
-      if (Vwarn_neg_dim_as_zero)
-	warning ("%s: converting negative dimension to zero", warnfor);
+      warning_with_id ("Octave:neg-dim-as-zero",
+		       "%s: converting negative dimension to zero", warnfor);
 
       nr = (nr < 0) ? 0 : nr;
       nc = (nc < 0) ? 0 : nc;
@@ -782,8 +819,9 @@ check_dimensions (dim_vector& dim, const char *warnfor)
         }
     }
 
-  if (neg && Vwarn_neg_dim_as_zero)
-    warning ("%s: converting negative dimension to zero", warnfor);
+  if (neg)
+    warning_with_id ("Octave:neg-dim-as-zero",
+		     "%s: converting negative dimension to zero", warnfor);
 }
 
 
@@ -932,9 +970,9 @@ octave_vformat (std::ostream& os, const char *fmt, va_list args)
   return retval;
 }
 
-/* XXX FIXME XXX -- we really need a configure test for this.  */
+/* FIXME -- we really need a configure test for this.  */
 
-#if defined __GNUC__ && __GNUC__ >= 3
+#if defined __GNUC__ && __GNUC__ >= 3 && ! defined __MINGW32__
 #define HAVE_C99_VSNPRINTF 1
 #endif
 
@@ -1081,37 +1119,13 @@ octave_sleep (double seconds)
       unsigned int sec
 	= (t > UINT_MAX) ? UINT_MAX : static_cast<unsigned int> (t);
 
+      // Versions of these functions that accept unsigned int args are
+      // defined in cutils.c.
       octave_sleep (sec);
       octave_usleep (usec);
     }
 }
 
-static int
-warn_neg_dim_as_zero (void)
-{
-  Vwarn_neg_dim_as_zero = check_preference ("warn_neg_dim_as_zero");
-
-  return 0;
-}
-
-void
-symbols_of_utils (void)
-{
-  DEFVAR (warn_neg_dim_as_zero, false, warn_neg_dim_as_zero,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} warn_neg_dim_as_zero\n\
-If the value of @code{warn_neg_dim_as_zero} is nonzero, print a warning\n\
-for expressions like\n\
-\n\
-@example\n\
-eye (-1)\n\
-@end example\n\
-\n\
-@noindent\n\
-The default value is 0.\n\
-@end defvr");
-}
-
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
