@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2002,
+              2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -58,6 +58,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "unwind-prot.h"
 #include "utils.h"
 #include "variables.h"
+#include "debug.h"
 
 // Should Octave always check to see if function files have changed
 // since they were last compiled?
@@ -906,8 +907,9 @@ unique_symbol_name (const std::string& basename)
 DEFUN (exist, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} exist (@var{name}, @var{type})\n\
-Return 1 if the name exists as a variable, 2 if the name (after\n\
-appending @samp{.m}) is a function file in Octave's @code{path}, 3 if the\n\
+Return 1 if the name exists as a variable, 2 if the name is an\n\
+absolute file name, an ordinary file in Octave's @code{path}, or (after\n\
+appending @samp{.m}) a function file in Octave's @code{path}, 3 if the\n\
 name is a @samp{.oct} or @samp{.mex} file in Octave's @code{path},\n\
 5 if the name is a built-in function, 7 if the name is a directory, or 103\n\
 if the name is a function not associated with a file (entered on\n\
@@ -1008,7 +1010,8 @@ symbol_out_of_date (symbol_record *sr)
 	      if (tc < Vlast_prompt_time
 		  || (relative && tc < Vlast_chdir_time))
 		{
-		  time_t tp = fcn->time_parsed ();
+		  octave_time ottp = fcn->time_parsed ();
+		  time_t tp = ottp.unix_time ();
 
 		  std::string nm = fcn->is_nested_function ()
 		    ? fcn->parent_fcn_name () : fcn->name ();
@@ -1089,6 +1092,11 @@ symbol_out_of_date (symbol_record *sr)
 		      else
 			retval = true;
 		    }
+
+		  // If the function has been replaced then clear any 
+		  // breakpoints associated with it
+		  if (retval)
+		    bp_table::remove_all_breakpoints_in_file (nm, true);
 		}
 	    }
 	}
@@ -1996,7 +2004,7 @@ then unlock the current function.\n\
       octave_user_function *fcn = octave_call_stack::caller_user_function ();
 
       if (fcn)
-        mlock (fcn->name ());
+        munlock (fcn->name ());
       else
         error ("munlock: invalid use outside a function");
     }
@@ -2391,7 +2399,7 @@ clear_symbol (const std::string& nm)
 
 DEFCMD (clear, args, ,
   "-*- texinfo -*-\n\
-@deffn {Command} clear [-x] pattern @dots{}\n\
+@deffn {Command} clear [options] pattern @dots{}\n\
 Delete the names matching the given patterns from the symbol table.  The\n\
 pattern may contain the following special characters:\n\
 \n\
@@ -2429,7 +2437,24 @@ will clear the variable definition and restore the definition of\n\
 @code{foo} as a function.  Executing @kbd{clear foo} a second time will\n\
 clear the function definition.\n\
 \n\
-With -x, clear the variables that don't match the patterns.\n\
+The following options are available in both long and short form\n\
+@table @code\n\
+@item -all, -a\n\
+Clears all local and global user-defined variables and all functions\n\
+from the symbol table.\n\
+\n\
+@item -exclusive, -x\n\
+Clears the variables that don't match the following pattern.\n\
+\n\
+@item -functions, -f\n\
+Clears the function names and the built-in symbols names.\n\
+@item -global, -g\n\
+Clears the global symbol names.\n\
+@item -variables, -v\n\
+Clears the local variable names.\n\
+@end table\n\
+With the execption of @code{exclusive}, all long options can be used \n\
+without the dash as well.\n\
 @end deffn")
 {
   octave_value_list retval;
@@ -2541,7 +2566,7 @@ With -x, clear the variables that don't match the patterns.\n\
 DEFUN (__print_symtab_info__, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} __print_symtab_info__ ()\n\
-Print raw symbol table statistices.\n\
+Undocumented internal function.\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -2587,7 +2612,7 @@ Print raw symbol table statistices.\n\
 DEFUN (__print_symbol_info__, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} __dump_symbol_info__ (@var{name})\n\
-Print symbol table information for the symbol @var{name}.\n\
+Undocumented internal function.\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -2631,7 +2656,7 @@ search path if they change.  If set to @code{\"all\"}, Octave will not\n\
 recompile any function files unless their definitions are removed with\n\
 @code{clear}.  If set to \"none\", Octave will always check time stamps\n\
 on files to determine whether functions defined in function files\n\
-need to recompiled.\n\
+need to be recompiled.\n\
 @end deftypefn")
 {
   octave_value retval;
